#!/usr/bin/env node
"use strict";
/* -*- typescript -*- */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_line_args_1 = __importDefault(require("command-line-args"));
const fs_1 = __importDefault(require("fs"));
const _1 = __importDefault(require("."));
// eslint-disable-next-line @typescript-eslint/no-var-requires
const { version } = require('../package.json');
async function main({ files, ...opts }) {
    let text = null;
    if (opts.version) {
        console.error('pandiff', version);
    }
    else if (files && files.length === 1 && files[0].endsWith('.docx')) {
        text = await _1.default.trackChanges(files[0], opts);
    }
    else if (files && files.length === 1 && files[0].endsWith('.md')) {
        text = fs_1.default.readFileSync(files[0], 'utf8');
        text = await _1.default.normalise(text, opts);
    }
    else if (files && files.length === 2) {
        const [file1, file2] = files;
        text = await (0, _1.default)(file1, file2, { ...opts, files: true });
    }
    else {
        help();
    }
    if (text)
        process.stdout.write(text);
}
const File = (s) => s;
const Format = (s) => s;
const Path = (s) => s;
const optionDefinitions = [
    { name: 'bibliography', type: File, multiple: true },
    { name: 'columns', type: Number },
    { name: 'csl', type: File, multiple: true },
    { name: 'extract-media', type: Path },
    { name: 'filter', alias: 'F', type: String, multiple: true },
    { name: 'from', alias: 'f', type: Format },
    { name: 'help', alias: 'h', type: Boolean },
    { name: 'highlight-style', type: String },
    { name: 'lua-filter', type: File, multiple: true },
    { name: 'mathjax', type: Boolean },
    { name: 'mathml', type: Boolean },
    { name: 'output', alias: 'o', type: File },
    { name: 'pdf-engine', type: String },
    { name: 'reference-doc', type: File, multiple: true },
    { name: 'reference-links', type: Boolean },
    { name: 'resource-path', type: Path },
    { name: 'standalone', alias: 's', type: Boolean },
    { name: 'to', alias: 't', type: Format },
    { name: 'version', alias: 'v', type: Boolean },
    { name: 'wrap', type: String },
    { name: 'files', multiple: true, defaultOption: true },
];
function help() {
    console.error('Usage: pandiff [OPTIONS] FILE1 FILE2');
    for (const opt of optionDefinitions) {
        if (opt.defaultOption)
            continue;
        let line = '  ';
        if (opt.alias) {
            line += `-${opt.alias},`;
        }
        else {
            line += '   ';
        }
        line += ` --${opt.name}`;
        if (opt.type && opt.type !== Boolean) {
            line += '=' + opt.type.name.toUpperCase();
        }
        console.error(line);
    }
}
main((0, command_line_args_1.default)(optionDefinitions)).catch(err => {
    console.error('Error:', err);
});
//# sourceMappingURL=cli.js.map